Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/w1/w1_int.c     |   17 ++++++-----------
 drivers/w1/w1_netlink.c |   26 ++++++++++++++++++++++++++
 drivers/w1/w1_netlink.h |    2 ++
 3 files changed, 34 insertions(+), 11 deletions(-)

diff -puN drivers/w1/w1_int.c~gregkh-i2c-w1-netlink-callbacks drivers/w1/w1_int.c
--- devel/drivers/w1/w1_int.c~gregkh-i2c-w1-netlink-callbacks	2005-08-31 17:45:45.000000000 -0700
+++ devel-akpm/drivers/w1/w1_int.c	2005-08-31 17:46:23.000000000 -0700
@@ -88,17 +88,13 @@ static struct w1_master * w1_alloc_dev(u
 
 	dev->groups = 1;
 	dev->seq = 1;
-	dev->nls = netlink_kernel_create(NETLINK_W1, 1, NULL, THIS_MODULE);
-	if (!dev->nls) {
-		printk(KERN_ERR "Failed to create new netlink socket(%u) for w1 master %s.\n",
-			NETLINK_NFLOG, dev->dev.bus_id);
-	}
+	dev_init_netlink(dev);
 
 	err = device_register(&dev->dev);
 	if (err) {
-		printk(KERN_ERR "Failed to register master device. err=%d\n", err);
-		if (dev->nls && dev->nls->sk_socket)
-			sock_release(dev->nls->sk_socket);
+		printk(KERN_ERR "Failed to register master device. err=%d\n",
+				err);
+		dev_fini_netlink(dev);
 		memset(dev, 0, sizeof(struct w1_master));
 		kfree(dev);
 		dev = NULL;
@@ -107,11 +103,10 @@ static struct w1_master * w1_alloc_dev(u
 	return dev;
 }
 
-static void w1_free_dev(struct w1_master *dev)
+void w1_free_dev(struct w1_master *dev)
 {
 	device_unregister(&dev->dev);
-	if (dev->nls && dev->nls->sk_socket)
-		sock_release(dev->nls->sk_socket);
+	dev_fini_netlink(dev);
 	memset(dev, 0, sizeof(struct w1_master) + sizeof(struct w1_bus_master));
 	kfree(dev);
 }
diff -puN drivers/w1/w1_netlink.c~gregkh-i2c-w1-netlink-callbacks drivers/w1/w1_netlink.c
--- devel/drivers/w1/w1_netlink.c~gregkh-i2c-w1-netlink-callbacks	2005-08-31 17:45:45.000000000 -0700
+++ devel-akpm/drivers/w1/w1_netlink.c	2005-08-31 17:45:45.000000000 -0700
@@ -57,10 +57,36 @@ void w1_netlink_send(struct w1_master *d
 nlmsg_failure:
 	return;
 }
+
+int dev_init_netlink(struct w1_master *dev)
+{
+	dev->nls = netlink_kernel_create(NETLINK_W1, NULL);
+	if (!dev->nls) {
+		printk(KERN_ERR "Failed to create new netlink socket(%u) for w1 master %s.\n",
+			NETLINK_W1, dev->dev.bus_id);
+	}
+
+	return 0;
+}
+
+void dev_fini_netlink(struct w1_master *dev)
+{
+	if (dev->nls && dev->nls->sk_socket)
+		sock_release(dev->nls->sk_socket);
+}
 #else
 #warning Netlink support is disabled. Please compile with NET support enabled.
 
 void w1_netlink_send(struct w1_master *dev, struct w1_netlink_msg *msg)
 {
 }
+
+int dev_init_netlink(struct w1_master *dev)
+{
+	return 0;
+}
+
+void dev_fini_netlink(struct w1_master *dev)
+{
+}
 #endif
diff -puN drivers/w1/w1_netlink.h~gregkh-i2c-w1-netlink-callbacks drivers/w1/w1_netlink.h
--- devel/drivers/w1/w1_netlink.h~gregkh-i2c-w1-netlink-callbacks	2005-08-31 17:45:45.000000000 -0700
+++ devel-akpm/drivers/w1/w1_netlink.h	2005-08-31 17:45:45.000000000 -0700
@@ -52,6 +52,8 @@ struct w1_netlink_msg
 #ifdef __KERNEL__
 
 void w1_netlink_send(struct w1_master *, struct w1_netlink_msg *);
+int dev_init_netlink(struct w1_master *dev);
+void dev_fini_netlink(struct w1_master *dev);
 
 #endif /* __KERNEL__ */
 #endif /* __W1_NETLINK_H */
_
